/*!\file window.c
 * \brief GL4Dummies, exemple simple 2D avec GL4Dummies
 * \author Farès BELHADJ, amsi@ai.univ-paris8.fr
 * \date February 04 2018, modified on March 06, 2020 by 
 * adding Visual Studio configuration files.
 */

/* inclusion des entêtes de fonctions de gestion de primitives simples
 * de dessin. La lettre p signifie aussi bien primitive que
 * pédagogique. */
#include <GL4D/gl4dp.h>

/* inclusion des entêtes de fonctions de création et de gestion de
 * fenêtres système ouvrant un contexte favorable à GL4dummies. Cette
 * partie est dépendante de la bibliothèque SDL2 */
#include <GL4D/gl4duw_SDL2.h>

/*!\brief identifiant du (futur) vertex array object */
static GLuint _vao = 0;
/*!\brief identifiant du (futur) buffer de data */
static GLuint _vbo = 0;

static GLuint _pid = 0;
static GLint _color = 0;


/*!\brief fonction appelée à chaque draw par la gl4duwMainLoop. */
static void dessin(void) {
  glClear(GL_COLOR_BUFFER_BIT);
  glUseProgram(_pid);
  glUniform3f(_color, 1, 1, 1);
  glBindVertexArray(_vao);
  glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 
}

/*!\brief Cette fonction est appelée au moment de sortir du programme
 *  (atexit), elle libère la fenêtre SDL \ref _win et le contexte
 *  OpenGL \ref _oglContext.
 */
static void quitte(void) {
  if(_vao)
    glDeleteVertexArrays(1, &_vao);
  if(_vbo)
    glDeleteBuffers(1, &_vbo);
  gl4duClean(GL4DU_ALL);
}



static void initData(void) {
  const char * imvs =
    "<imvs>sylvia_dummy.vs</imvs>\n"
    "#version 330\n"
    "layout (location = 0) in vec2 vsiPosition;\n"
    "void main(void) {\n"
    "gl_Position = vec4(vsiPosition, 0.0, 1.0);\n"
    "}\n";
  const char * imfs =
    "<imfs>sylvia_dummy.fs</imfs>\n"
    "#version 330\n"
    "uniform vec3 color;\n"
    "out vec4 fragColor;\n"
    "void main(void) {\n"
    "fragColor = vec4(color,1);\n"
    "}";
  _pid = gl4duCreateProgram(imvs, imfs, NULL);

  _color = glGetUniformLocation (_pid, "color");
  
  GLfloat data [] = {
    /* 4 coordonnées de sommets du carré centré en 0, 0 et de côté 150*/
    -0.25f, -0.25f,
     0.25f, -0.25f,
    -0.25f,  0.25f,
     0.25f,  0.25f
  };

    glGenVertexArrays (1, &_vao);
   /* Activation du VAO*/
  glBindVertexArray(_vao);

  glGenBuffers(1, &_vbo);
  glBindBuffer (GL_ARRAY_BUFFER, _vbo);
  glBufferData(GL_ARRAY_BUFFER, sizeof data, data, GL_STATIC_DRAW);
  glEnableVertexAttribArray(0);
  glVertexAttribPointer (0, 2, GL_FLOAT, GL_FALSE, 0, 0); 
  /* Désactivation du VAO */
  glBindVertexArray(0);

  glClearColor(0.9f, 0.2f, 0.2f, 1.0f);  
  /* dans quelle partie de l'écran on effectue le rendu */
  glViewport(0, 0, 600, 600);

}



/*!\brief créé la fenêtre, un screen 2D effacé en noir et lance une
 *  boucle infinie.*/
int main(int argc, char ** argv) {
  /* tentative de création d'une fenêtre pour GL4Dummies */
  if(!gl4duwCreateWindow(argc, argv, /* args du programme */
			 "GL4Dummies' Hello World", /* titre */
			 10, 10, 600, 600, /* x,y, largeur, heuteur */
			 GL4DW_SHOWN) /* état visible */) {
    /* ici si échec de la création souvent lié à un problème d'absence
     * de contexte graphique ou d'impossibilité d'ouverture d'un
     * contexte OpenGL (au moins 3.2) */
    return 1;
  }
  /* création d'un screen GL4Dummies (texture dans laquelle nous
   * pouvons dessiner) aux dimensions de la fenêtre */
  gl4dpInitScreen();
  initData (); 
  /* ajoute la fonction quitte à la pile des choses à faire en sortant
   * du programme */
  atexit(quitte);
  /* met en place une fonction de display au sein de la boucle
   * event-simu-draw gl4duwMainLoop */
  gl4duwDisplayFunc(dessin);
  /* boucle infinie pour éviter que le programme ne s'arrête et ferme
   * la fenêtre */
  gl4duwMainLoop();
  return 0;

}



